@@ -1,11 +1,12 @@
-/* $Id: minissdp.c,v 1.36 2012/05/27 22:16:26 nanard Exp $ */
+/* $Id: minissdp.c,v 1.55 2013/07/12 05:24:19 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2012 Thomas Bernard
+ * (c) 2006-2013 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/socket.h>
@@ -14,6 +15,7 @@
 #include <arpa/inet.h>
 #include <errno.h>
 #include <syslog.h>
+
 #include "config.h"
 #include "upnpdescstrings.h"
 #include "miniupnpdpath.h"
@@ -21,6 +23,7 @@
 #include "upnpglobalvars.h"
 #include "minissdp.h"
 #include "upnputils.h"
+#include "getroute.h"
 #include "codelength.h"
 
 /* SSDP ip/port */
@@ -310,17 +313,36 @@ USN:uuid:upnp-InternetGatewayDevice-1_0-0090a2777777::upnp:rootdevice
 EXT:
  */
 
-/* not really an SSDP "announce" as it is the response
- * to a SSDP "M-SEARCH" */
+/* Responds to a SSDP "M-SEARCH"
+ * s :          socket to use
+ * addr :       peer
+ * st, st_len : ST: header
+ * suffix :     suffix for USN: header
+ * host, port : our HTTP host, port
+ */
 static void
-SendSSDPAnnounce2(int s, const struct sockaddr * addr,
-                  const char * st, int st_len, const char * suffix,
-                  const char * host, unsigned short port)
+SendSSDPResponse(int s, const struct sockaddr * addr,
+                 const char * st, int st_len, const char * suffix,
+                 const char * host, unsigned short port, const char * uuidvalue)
 {
 	int l, n;
 	char buf[512];
 	char addr_str[64];
 	socklen_t addrlen;
+	int st_is_uuid;
+#ifdef ENABLE_HTTP_DATE
+	char http_date[64];
+	time_t t;
+	struct tm tm;
+
+	time(&t);
+	gmtime_r(&t, &tm);
+	strftime(http_date, sizeof(http_date),
+		    "%a, %d %b %Y %H:%M:%S GMT", &tm);
+#endif
+
+	st_is_uuid = (st_len == (int)strlen(uuidvalue)) &&
+	              (memcmp(uuidvalue, st, st_len) == 0);
 	/*
 	 * follow guideline from document "UPnP Device Architecture 1.0"
 	 * uppercase is recommended.
@@ -331,9 +353,11 @@ SendSSDPAnnounce2(int s, const struct sockaddr * addr,
 	 * have a look at the document "UPnP Device Architecture v1.1 */
 	l = snprintf(buf, sizeof(buf), "HTTP/1.1 200 OK\r\n"
 		"CACHE-CONTROL: max-age=120\r\n"
-		/*"DATE: ...\r\n"*/
+#ifdef ENABLE_HTTP_DATE
+		"DATE: %s\r\n"
+#endif
 		"ST: %.*s%s\r\n"
-		"USN: %s::%.*s%s\r\n"
+		"USN: %s%s%.*s%s\r\n"
 		"EXT:\r\n"
 		"SERVER: " MINIUPNPD_SERVER_STRING "\r\n"
 		"LOCATION: http://%s:%u" ROOTDESC_PATH "\r\n"
@@ -342,10 +366,26 @@ SendSSDPAnnounce2(int s, const struct sockaddr * addr,
 		"BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
 		"CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
 		"\r\n",
+#ifdef ENABLE_HTTP_DATE
+		http_date,
+#endif
 		st_len, st, suffix,
-		uuidvalue, st_len, st, suffix,
+		uuidvalue, st_is_uuid ? "" : "::",
+		st_is_uuid ? 0 : st_len, st, suffix,
 		host, (unsigned int)port,
 		upnp_bootid, upnp_bootid, upnp_configid);
+	if(l<0)
+	{
+		syslog(LOG_ERR, "%s: snprintf failed %m",
+		       "SendSSDPResponse()");
+		return;
+	}
+	else if((unsigned)l>=sizeof(buf))
+	{
+		syslog(LOG_WARNING, "%s: truncated output",
+		       "SendSSDPResponse()");
+		l = sizeof(buf) - 1;
+	}
 	addrlen = (addr->sa_family == AF_INET6)
 	          ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr_in);
 	n = sendto(s, buf, l, 0,
@@ -361,24 +401,101 @@ SendSSDPAnnounce2(int s, const struct sockaddr * addr,
 	}
 }
 
-static const char * const known_service_types[] =
+#ifndef IGD_V2
+#define IGD_VER 1
+#define WANIPC_VER 1
+#else
+#define IGD_VER 2
+#define WANIPC_VER 2
+#endif
+
+static struct {
+	const char * s;
+	const int version;
+	const char * uuid;
+} const known_service_types[] =
 {
-	"upnp:rootdevice",
-	"urn:schemas-upnp-org:device:InternetGatewayDevice:",
-	"urn:schemas-upnp-org:device:WANConnectionDevice:",
-	"urn:schemas-upnp-org:device:WANDevice:",
-	"urn:schemas-upnp-org:service:WANCommonInterfaceConfig:",
-	"urn:schemas-upnp-org:service:WANIPConnection:",
-	"urn:schemas-upnp-org:service:WANPPPConnection:",
+	{"upnp:rootdevice", 0, uuidvalue_igd},
+	{"urn:schemas-upnp-org:device:InternetGatewayDevice:", IGD_VER, uuidvalue_igd},
+	{"urn:schemas-upnp-org:device:WANConnectionDevice:", 1, uuidvalue_wcd},
+	{"urn:schemas-upnp-org:device:WANDevice:", 1, uuidvalue_wan},
+	{"urn:schemas-upnp-org:service:WANCommonInterfaceConfig:", 1, uuidvalue_wan},
+	{"urn:schemas-upnp-org:service:WANIPConnection:", WANIPC_VER, uuidvalue_wcd},
+#ifndef UPNP_STRICT
+	/* We use WAN IP Connection, not PPP connection,
+	 * but buggy control points may try to use WanPPPConnection
+	 * anyway */
+	{"urn:schemas-upnp-org:service:WANPPPConnection:", 1, uuidvalue_wcd},
+#endif
 #ifdef ENABLE_L3F_SERVICE
-	"urn:schemas-upnp-org:service:Layer3Forwarding:",
+	{"urn:schemas-upnp-org:service:Layer3Forwarding:", 1, uuidvalue_igd},
 #endif
 #ifdef ENABLE_6FC_SERVICE
-	"url:schemas-upnp-org:service:WANIPv6FirewallControl:",
+	{"url:schemas-upnp-org:service:WANIPv6FirewallControl:", 1, uuidvalue_wcd},
 #endif
-	0
+	{0, 0, 0}
 };
 
+static void
+SendSSDPNotify(int s, const struct sockaddr * dest,
+               const char * host, unsigned short port,
+               const char * nt, const char * suffix,
+               const char * usn1, const char * usn2, const char * usn3,
+               unsigned int lifetime, int ipv6)
+{
+	char bufr[512];
+	int n, l;
+
+	l = snprintf(bufr, sizeof(bufr),
+		"NOTIFY * HTTP/1.1\r\n"
+		"HOST: %s:%d\r\n"
+		"CACHE-CONTROL: max-age=%u\r\n"
+		"LOCATION: http://%s:%d" ROOTDESC_PATH"\r\n"
+		"SERVER: " MINIUPNPD_SERVER_STRING "\r\n"
+		"NT: %s%s\r\n"
+		"USN: %s%s%s%s\r\n"
+		"NTS: ssdp:alive\r\n"
+		"OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n" /* UDA v1.1 */
+		"01-NLS: %u\r\n" /* same as BOOTID field. UDA v1.1 */
+		"BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
+		"CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
+		"\r\n",
+		ipv6 ? "[" LL_SSDP_MCAST_ADDR "]" : SSDP_MCAST_ADDR,
+		SSDP_PORT,
+		lifetime,
+		host, port,
+		nt, suffix, /* NT: */
+		usn1, usn2, usn3, suffix, /* USN: */
+		upnp_bootid, upnp_bootid, upnp_configid );
+	if(l<0)
+	{
+		syslog(LOG_ERR, "SendSSDPNotify() snprintf error");
+		return;
+	}
+	else if((unsigned int)l >= sizeof(bufr))
+	{
+		syslog(LOG_WARNING, "SendSSDPNotify(): truncated output");
+		l = sizeof(bufr) - 1;
+	}
+	n = sendto(s, bufr, l, 0, dest,
+#ifdef ENABLE_IPV6
+		ipv6 ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr_in)
+#else
+		sizeof(struct sockaddr_in)
+#endif
+		);
+	if(n < 0)
+	{
+		/* XXX handle EINTR, EAGAIN, EWOULDBLOCK */
+		syslog(LOG_ERR, "sendto(udp_notify=%d, %s): %m", s,
+		       host ? host : "NULL");
+	}
+	else if(n != l)
+	{
+		syslog(LOG_NOTICE, "sendto() sent %d out of %d bytes", n, l);
+	}
+}
+
 static void
 SendSSDPNotifies(int s, const char * host, unsigned short port,
                  unsigned int lifetime, int ipv6)
@@ -388,8 +505,8 @@ SendSSDPNotifies(int s, const char * host, unsigned short port,
 #else
 	struct sockaddr_in sockname;
 #endif
-	int l, n, i=0;
-	char bufr[512];
+	int i=0;
+	char ver_str[4];
 
 	memset(&sockname, 0, sizeof(sockname));
 #ifdef ENABLE_IPV6
@@ -409,52 +526,24 @@ SendSSDPNotifies(int s, const char * host, unsigned short port,
 		p->sin_addr.s_addr = inet_addr(SSDP_MCAST_ADDR);
 	}
 
-	while(known_service_types[i])
+	while(known_service_types[i].s)
 	{
-		l = snprintf(bufr, sizeof(bufr),
-			"NOTIFY * HTTP/1.1\r\n"
-			"HOST: %s:%d\r\n"
-			"CACHE-CONTROL: max-age=%u\r\n"
-			"lOCATION: http://%s:%d" ROOTDESC_PATH"\r\n"
-			"SERVER: " MINIUPNPD_SERVER_STRING "\r\n"
-			"NT: %s%s\r\n"
-			"USN: %s::%s%s\r\n"
-			"NTS: ssdp:alive\r\n"
-			"OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n" /* UDA v1.1 */
-			"01-NLS: %u\r\n" /* same as BOOTID field. UDA v1.1 */
-			"BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
-			"CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
-			"\r\n",
-			ipv6 ? "[" LL_SSDP_MCAST_ADDR "]" : SSDP_MCAST_ADDR,
-			SSDP_PORT,
-			lifetime,
-			host, port,
-			known_service_types[i], (i==0?"":"1"),
-			uuidvalue, known_service_types[i], (i==0?"":"1"),
-			upnp_bootid, upnp_bootid, upnp_configid );
-		if(l<0)
+		if(i==0)
+			ver_str[0] = '\0';
+		else
+			snprintf(ver_str, sizeof(ver_str), "%d", known_service_types[i].version);
+		SendSSDPNotify(s, (struct sockaddr *)&sockname, host, port,
+		               known_service_types[i].s, ver_str,	/* NT: */
+		               known_service_types[i].uuid, "::",
+		               known_service_types[i].s, /* ver_str,	USN: */
+		               lifetime, ipv6);
+		if(0==memcmp(known_service_types[i].s,
+		             "urn:schemas-upnp-org:device", sizeof("urn:schemas-upnp-org:device")-1))
 		{
-			syslog(LOG_ERR, "SendSSDPNotifies() snprintf error");
-			continue;
-		}
-		if((unsigned int)l >= sizeof(bufr))
-		{
-			syslog(LOG_WARNING, "SendSSDPNotifies(): truncated output");
-			l = sizeof(bufr);
-		}
-		n = sendto(s, bufr, l, 0,
-			(struct sockaddr *)&sockname,
-#ifdef ENABLE_IPV6
-			ipv6 ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr_in)
-#else
-			sizeof(struct sockaddr_in)
-#endif
-			);
-		if(n < 0)
-		{
-			/* XXX handle EINTR, EAGAIN, EWOULDBLOCK */
-			syslog(LOG_ERR, "sendto(udp_notify=%d, %s): %m", s,
-			       host ? host : "NULL");
+			SendSSDPNotify(s, (struct sockaddr *)&sockname, host, port,
+			               known_service_types[i].uuid, "",	/* NT: */
+			               known_service_types[i].uuid, "", "", /* ver_str,	USN: */
+			               lifetime, ipv6);
 		}
 		i++;
 	}
@@ -523,11 +612,26 @@ ProcessSSDPData(int s, const char *bufr, int n,
 	struct lan_addr_s * lan_addr = NULL;
 	const char * st = NULL;
 	int st_len = 0;
+	int st_ver = 0;
 	char sender_str[64];
+	char ver_str[4];
 	const char * announced_host = NULL;
+#ifdef UPNP_STRICT
+#ifdef ENABLE_IPV6
+	char announced_host_buf[64];
+#endif
+	int mx_value = -1;
+#endif
 
 	/* get the string representation of the sender address */
 	sockaddr_to_string(sender, sender_str, sizeof(sender_str));
+	lan_addr = get_lan_for_peer(sender);
+	if(lan_addr == NULL)
+	{
+		syslog(LOG_WARNING, "SSDP packet sender %s not from a LAN, ignoring",
+		       sender_str);
+		return;
+	}
 
 	if(memcmp(bufr, "NOTIFY", 6) == 0)
 	{
@@ -551,12 +655,38 @@ ProcessSSDPData(int s, const char *bufr, int n,
 				while(st[st_len]!='\r' && st[st_len]!='\n'
 				     && (st + st_len < bufr + n))
 					st_len++;
-				/*syslog(LOG_INFO, "ST: %.*s", st_len, st);*/
+				l = st_len;
+				while(l > 0 && st[l-1] != ':')
+					l--;
+				st_ver = atoi(st+l);
+				syslog(LOG_DEBUG, "ST: %.*s (ver=%d)", st_len, st, st_ver);
 				/*j = 0;*/
 				/*while(bufr[i+j]!='\r') j++;*/
 				/*syslog(LOG_INFO, "%.*s", j, bufr+i);*/
 			}
+#ifdef UPNP_STRICT
+			else if((i < n - 3) && (strncasecmp(bufr+i, "mx:", 3) == 0))
+			{
+				const char * mx;
+				int mx_len;
+				mx = bufr+i+3;
+				mx_len = 0;
+				while((*mx == ' ' || *mx == '\t') && (mx < bufr + n))
+					mx++;
+				while(mx[mx_len]!='\r' && mx[mx_len]!='\n'
+				     && (mx + mx_len < bufr + n))
+					mx_len++;
+				mx_value = atoi(mx);
+				syslog(LOG_DEBUG, "MX: %.*s (value=%d)", mx_len, mx, mx_value);
+			}
+#endif
 		}
+#ifdef UPNP_STRICT
+		if(mx_value < 0) {
+			syslog(LOG_INFO, "ignoring SSDP packet missing MX: header");
+			return;
+		}
+#endif
 		/*syslog(LOG_INFO, "SSDP M-SEARCH packet received from %s",
 	           sender_str );*/
 		if(st && (st_len > 0))
@@ -567,14 +697,6 @@ ProcessSSDPData(int s, const char *bufr, int n,
 			/* find in which sub network the client is */
 			if(sender->sa_family == AF_INET)
 			{
-				for(lan_addr = lan_addrs.lh_first;
-				    lan_addr != NULL;
-				    lan_addr = lan_addr->list.le_next)
-				{
-					if( (((const struct sockaddr_in *)sender)->sin_addr.s_addr & lan_addr->mask.s_addr)
-					   == (lan_addr->addr.s_addr & lan_addr->mask.s_addr))
-						break;
-				}
 				if (lan_addr == NULL)
 				{
 					syslog(LOG_ERR, "Can't find in which sub network the client is");
@@ -586,19 +708,56 @@ ProcessSSDPData(int s, const char *bufr, int n,
 			else
 			{
 				/* IPv6 address with brackets */
+#ifdef UPNP_STRICT
+				int index;
+				struct in6_addr addr6;
+				size_t addr6_len = sizeof(addr6);
+				/* retrieve the IPv6 address which
+				 * will be used locally to reach sender */
+				memset(&addr6, 0, sizeof(addr6));
+				if(get_src_for_route_to (sender, &addr6, &addr6_len, &index) < 0) {
+					syslog(LOG_WARNING, "get_src_for_route_to() failed, using %s", ipv6_addr_for_http_with_brackets);
+					announced_host = ipv6_addr_for_http_with_brackets;
+				} else {
+					if(inet_ntop(AF_INET6, &addr6,
+					             announced_host_buf+1,
+					             sizeof(announced_host_buf) - 2)) {
+						announced_host_buf[0] = '[';
+						i = strlen(announced_host_buf);
+						if(i < (int)sizeof(announced_host_buf) - 1) {
+							announced_host_buf[i] = ']';
+							announced_host_buf[i+1] = '\0';
+						} else {
+							syslog(LOG_NOTICE, "cannot suffix %s with ']'",
+							       announced_host_buf);
+						}
+						announced_host = announced_host_buf;
+					} else {
+						syslog(LOG_NOTICE, "inet_ntop() failed %m");
+						announced_host = ipv6_addr_for_http_with_brackets;
+					}
+				}
+#else
 				announced_host = ipv6_addr_for_http_with_brackets;
+#endif
 			}
 #endif
 			/* Responds to request with a device as ST header */
-			for(i = 0; known_service_types[i]; i++)
+			for(i = 0; known_service_types[i].s; i++)
 			{
-				l = (int)strlen(known_service_types[i]);
-				if(l<=st_len && (0 == memcmp(st, known_service_types[i], l)))
+				l = (int)strlen(known_service_types[i].s);
+				if(l<=st_len && (0 == memcmp(st, known_service_types[i].s, l))
+#ifdef UPNP_STRICT
+				   && (st_ver <= known_service_types[i].version)
+		/* only answer for service version lower or equal of supported one */
+#endif
+				   )
 				{
 					syslog(LOG_INFO, "Single search found");
-					SendSSDPAnnounce2(s, sender,
-					                  st, st_len, "",
-					                  announced_host, port);
+					SendSSDPResponse(s, sender,
+					                 st, st_len, "",
+					                 announced_host, port,
+					                 known_service_types[i].uuid);
 					break;
 				}
 			}
@@ -607,21 +766,48 @@ ProcessSSDPData(int s, const char *bufr, int n,
 			if(st_len==8 && (0 == memcmp(st, "ssdp:all", 8)))
 			{
 				syslog(LOG_INFO, "ssdp:all found");
-				for(i=0; known_service_types[i]; i++)
+				for(i=0; known_service_types[i].s; i++)
 				{
-					l = (int)strlen(known_service_types[i]);
-					SendSSDPAnnounce2(s, sender,
-					                  known_service_types[i], l, i==0?"":"1",
-					                  announced_host, port);
+					if(i==0)
+						ver_str[0] = '\0';
+					else
+						snprintf(ver_str, sizeof(ver_str), "%d", known_service_types[i].version);
+					l = (int)strlen(known_service_types[i].s);
+					SendSSDPResponse(s, sender,
+					                 known_service_types[i].s, l, ver_str,
+					                 announced_host, port,
+					                 known_service_types[i].uuid);
 				}
+				/* also answer for uuid */
+				SendSSDPResponse(s, sender, uuidvalue_igd, strlen(uuidvalue_igd), "",
+				                 announced_host, port, uuidvalue_igd);
+				SendSSDPResponse(s, sender, uuidvalue_wan, strlen(uuidvalue_wan), "",
+				                 announced_host, port, uuidvalue_wan);
+				SendSSDPResponse(s, sender, uuidvalue_wcd, strlen(uuidvalue_wcd), "",
+				                 announced_host, port, uuidvalue_wcd);
 			}
 			/* responds to request by UUID value */
-			l = (int)strlen(uuidvalue);
-			if(l==st_len && (0 == memcmp(st, uuidvalue, l)))
+			l = (int)strlen(uuidvalue_igd);
+			if(l==st_len)
 			{
-				syslog(LOG_INFO, "ssdp:uuid found");
-				SendSSDPAnnounce2(s, sender, st, st_len, "",
-				                  announced_host, port);
+				if(0 == memcmp(st, uuidvalue_igd, l))
+				{
+					syslog(LOG_INFO, "ssdp:uuid (IGD) found");
+					SendSSDPResponse(s, sender, st, st_len, "",
+					                 announced_host, port, uuidvalue_igd);
+				}
+				else if(0 == memcmp(st, uuidvalue_wan, l))
+				{
+					syslog(LOG_INFO, "ssdp:uuid (WAN) found");
+					SendSSDPResponse(s, sender, st, st_len, "",
+					                 announced_host, port, uuidvalue_wan);
+				}
+				else if(0 == memcmp(st, uuidvalue_wcd, l))
+				{
+					syslog(LOG_INFO, "ssdp:uuid (WCD) found");
+					SendSSDPResponse(s, sender, st, st_len, "",
+					                 announced_host, port, uuidvalue_wcd);
+				}
 			}
 		}
 		else
@@ -635,6 +821,61 @@ ProcessSSDPData(int s, const char *bufr, int n,
 	}
 }
 
+static int
+SendSSDPbyebye(int s, const struct sockaddr * dest,
+               const char * nt, const char * suffix,
+               const char * usn1, const char * usn2, const char * usn3,
+               int ipv6)
+{
+	int n, l;
+	char bufr[512];
+
+	l = snprintf(bufr, sizeof(bufr),
+	             "NOTIFY * HTTP/1.1\r\n"
+	             "HOST: %s:%d\r\n"
+	             "NT: %s%s\r\n"
+	             "USN: %s%s%s%s\r\n"
+	             "NTS: ssdp:byebye\r\n"
+	             "OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n" /* UDA v1.1 */
+	             "01-NLS: %u\r\n" /* same as BOOTID field. UDA v1.1 */
+	             "BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
+	             "CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
+	             "\r\n",
+	             ipv6 ? "[" LL_SSDP_MCAST_ADDR "]" : SSDP_MCAST_ADDR,
+	             SSDP_PORT,
+	             nt, suffix,	/* NT: */
+	             usn1, usn2, usn3, suffix,	/* USN: */
+	             upnp_bootid, upnp_bootid, upnp_configid);
+	if(l<0)
+	{
+		syslog(LOG_ERR, "SendSSDPbyebye() snprintf error");
+		return -1;
+	}
+	else if((unsigned int)l >= sizeof(bufr))
+	{
+		syslog(LOG_WARNING, "SendSSDPbyebye(): truncated output");
+		l = sizeof(bufr) - 1;
+	}
+	n = sendto(s, bufr, l, 0, dest,
+#ifdef ENABLE_IPV6
+	           ipv6 ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr_in)
+#else
+	           sizeof(struct sockaddr_in)
+#endif
+	          );
+	if(n < 0)
+	{
+		syslog(LOG_ERR, "sendto(udp_shutdown=%d): %m", s);
+		return -1;
+	}
+	else if(n != l)
+	{
+		syslog(LOG_NOTICE, "sendto() sent %d out of %d bytes", n, l);
+		return -1;
+	}
+	return 0;
+}
+
 /* This will broadcast ssdp:byebye notifications to inform
  * the network that UPnP is going down. */
 int
@@ -644,9 +885,8 @@ SendSSDPGoodbye(int * sockets, int n_sockets)
 #ifdef ENABLE_IPV6
 	struct sockaddr_in6 sockname6;
 #endif
-	int n, l;
 	int i, j;
-	char bufr[512];
+	char ver_str[4];
 	int ret = 0;
 	int ipv6 = 0;
 
@@ -666,38 +906,34 @@ SendSSDPGoodbye(int * sockets, int n_sockets)
 #ifdef ENABLE_IPV6
 		ipv6 = j & 1;
 #endif
-	    for(i=0; known_service_types[i]; i++)
+	    for(i=0; known_service_types[i].s; i++)
 	    {
-	        l = snprintf(bufr, sizeof(bufr),
-                 "NOTIFY * HTTP/1.1\r\n"
-                 "HOST: %s:%d\r\n"
-                 "NT: %s%s\r\n"
-                 "USN: %s::%s%s\r\n"
-                 "NTS: ssdp:byebye\r\n"
-				 "OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n" /* UDA v1.1 */
-				 "01-NLS: %u\r\n" /* same as BOOTID field. UDA v1.1 */
-				 "BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
-				 "CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
-                 "\r\n",
-                 ipv6 ? "[" LL_SSDP_MCAST_ADDR "]" : SSDP_MCAST_ADDR,
-			     SSDP_PORT,
-				 known_service_types[i], (i==0?"":"1"),
-                 uuidvalue, known_service_types[i], (i==0?"":"1"),
-                 upnp_bootid, upnp_bootid, upnp_configid);
-	        n = sendto(sockets[j], bufr, l, 0,
+			if(i==0)
+				ver_str[0] = '\0';
+			else
+				snprintf(ver_str, sizeof(ver_str), "%d", known_service_types[i].version);
+			ret += SendSSDPbyebye(sockets[j],
 #ifdef ENABLE_IPV6
-	                   ipv6 ? (struct sockaddr *)&sockname6 : (struct sockaddr *)&sockname,
-	                   ipv6 ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr_in)
+			                      ipv6 ? (struct sockaddr *)&sockname6 : (struct sockaddr *)&sockname,
 #else
-	                   (struct sockaddr *)&sockname,
-	                   sizeof(struct sockaddr_in)
+			                      (struct sockaddr *)&sockname,
 #endif
-	                 );
-			if(n < 0)
+			                      known_service_types[i].s, ver_str,	/* NT: */
+			                      known_service_types[i].uuid, "::",
+			                      known_service_types[i].s, /* ver_str, USN: */
+			                      ipv6);
+			if(0==memcmp(known_service_types[i].s,
+			             "urn:schemas-upnp-org:device", sizeof("urn:schemas-upnp-org:device")-1))
 			{
-				syslog(LOG_ERR, "SendSSDPGoodbye: sendto(udp_shutdown=%d): %m",
-				       sockets[j]);
-				ret = -1;
+				ret += SendSSDPbyebye(sockets[j],
+#ifdef ENABLE_IPV6
+				                      ipv6 ? (struct sockaddr *)&sockname6 : (struct sockaddr *)&sockname,
+#else
+				                      (struct sockaddr *)&sockname,
+#endif
+				                      known_service_types[i].uuid, "",	/* NT: */
+				                      known_service_types[i].uuid, "", "", /* ver_str, USN: */
+				                      ipv6);
 			}
     	}
 	}
@@ -714,7 +950,8 @@ SubmitServicesToMiniSSDPD(const char * host, unsigned short port) {
 	unsigned char buffer[2048];
 	char strbuf[256];
 	unsigned char * p;
-	int i, l;
+	int i, l, n;
+	char ver_str[4];
 
 	s = socket(AF_UNIX, SOCK_STREAM, 0);
 	if(s < 0) {
@@ -725,21 +962,33 @@ SubmitServicesToMiniSSDPD(const char * host, unsigned short port) {
 	strncpy(addr.sun_path, minissdpdsocketpath, sizeof(addr.sun_path));
 	if(connect(s, (struct sockaddr *)&addr, sizeof(struct sockaddr_un)) < 0) {
 		syslog(LOG_ERR, "connect(\"%s\"): %m", minissdpdsocketpath);
+		close(s);
 		return -1;
 	}
-	for(i = 0; known_service_types[i]; i++) {
-		buffer[0] = 4;
+	for(i = 0; known_service_types[i].s; i++) {
+		buffer[0] = 4;	/* request type 4 : submit service */
+		/* 4 strings following : ST (service type), USN, Server, Location */
 		p = buffer + 1;
-		l = (int)strlen(known_service_types[i]);
+		l = (int)strlen(known_service_types[i].s);
 		if(i > 0)
 			l++;
 		CODELENGTH(l, p);
-		memcpy(p, known_service_types[i], l);
+		memcpy(p, known_service_types[i].s, l);
 		if(i > 0)
 			p[l-1] = '1';
 		p += l;
+		if(i==0)
+			ver_str[0] = '\0';
+		else
+			snprintf(ver_str, sizeof(ver_str), "%d", known_service_types[i].version);
 		l = snprintf(strbuf, sizeof(strbuf), "%s::%s%s",
-		             uuidvalue, known_service_types[i], (i==0)?"":"1");
+		             known_service_types[i].uuid, known_service_types[i].s, ver_str);
+		if(l<0) {
+			syslog(LOG_WARNING, "SubmitServicesToMiniSSDPD: snprintf %m");
+			continue;
+		} else if((unsigned)l>=sizeof(strbuf)) {
+			l = sizeof(strbuf) - 1;
+		}
 		CODELENGTH(l, p);
 		memcpy(p, strbuf, l);
 		p += l;
@@ -749,12 +998,31 @@ SubmitServicesToMiniSSDPD(const char * host, unsigned short port) {
 		p += l;
 		l = snprintf(strbuf, sizeof(strbuf), "http://%s:%u" ROOTDESC_PATH,
 		             host, (unsigned int)port);
+		if(l<0) {
+			syslog(LOG_WARNING, "SubmitServicesToMiniSSDPD: snprintf %m");
+			continue;
+		} else if((unsigned)l>=sizeof(strbuf)) {
+			l = sizeof(strbuf) - 1;
+		}
 		CODELENGTH(l, p);
 		memcpy(p, strbuf, l);
 		p += l;
-		if(write(s, buffer, p - buffer) < 0) {
-			syslog(LOG_ERR, "write(): %m");
-			return -1;
+		/* now write the encoded data */
+		n = p - buffer;	/* bytes to send */
+		p = buffer;	/* start */
+		while(n > 0) {
+			l = write(s, p, n);
+			if (l < 0) {
+				syslog(LOG_ERR, "write(): %m");
+				close(s);
+				return -1;
+			} else if (l == 0) {
+				syslog(LOG_ERR, "write() returned 0");
+				close(s);
+				return -1;
+			}
+			p += l;
+			n -= l;
 		}
 	}
  	close(s);
