@@ -1,7 +1,7 @@
-/* $Id: miniupnpd.c,v 1.161 2012/05/21 15:50:03 nanard Exp $ */
+/* $Id: miniupnpd.c,v 1.176 2013/06/13 13:21:29 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2012 Thomas Bernard
+ * (c) 2006-2013 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -384,12 +384,13 @@ write_ctlsockets_list(int fd, struct ctlelem * e)
 	}
 }
 
+#ifndef DISABLE_CONFIG_FILE
 static void
 write_option_list(int fd)
 {
 	char buffer[256];
 	int len;
-	int i;
+	unsigned int i;
 	write(fd, "Options :\n", 10);
 	for(i=0; i<num_options; i++)
 	{
@@ -399,6 +400,7 @@ write_option_list(int fd)
 		write(fd, buffer, len);
 	}
 }
+#endif
 
 static void
 write_command_line(int fd, int argc, char * * argv)
@@ -559,7 +561,8 @@ parselanaddr(struct lan_addr_s * lan_addr, const char * str)
 		/* not starting with a digit : suppose it is an interface name */
 		memcpy(lan_addr->ifname, str, n);
 		lan_addr->ifname[n] = '\0';
-		if(getifaddr(lan_addr->ifname, lan_addr->str, sizeof(lan_addr->str)) < 0)
+		if(getifaddr(lan_addr->ifname, lan_addr->str, sizeof(lan_addr->str),
+		             &lan_addr->addr, &lan_addr->mask) < 0)
 			goto parselan_error;
 	}
 	else
@@ -568,9 +571,9 @@ parselanaddr(struct lan_addr_s * lan_addr, const char * str)
 			goto parselan_error;
 		memcpy(lan_addr->str, str, n);
 		lan_addr->str[n] = '\0';
+		if(!inet_aton(lan_addr->str, &lan_addr->addr))
+			goto parselan_error;
 	}
-	if(!inet_aton(lan_addr->str, &lan_addr->addr))
-		goto parselan_error;
 	if(*p == '/')
 	{
 		const char * q = ++p;
@@ -596,7 +599,7 @@ parselanaddr(struct lan_addr_s * lan_addr, const char * str)
 			lan_addr->mask.s_addr = htonl(nbits ? (0xffffffffu << (32 - nbits)) : 0);
 		}
 	}
-	else
+	else if(lan_addr->mask.s_addr == 0)
 	{
 		/* by default, networks are /24 */
 		lan_addr->mask.s_addr = htonl(0xffffff00u);
@@ -628,6 +631,10 @@ parselanaddr(struct lan_addr_s * lan_addr, const char * str)
 			fprintf(stderr, "Cannot get index for network interface %s",
 			        lan_addr->ifname);
 	}
+	else
+	{
+		fprintf(stderr, "Warning: please specify LAN network interface by name instead of IPv4 address\n");
+	}
 #endif
 	return 0;
 parselan_error:
@@ -636,6 +643,35 @@ parselan_error:
 	return -1;
 }
 
+/* fill uuidvalue_wan and uuidvalue_wcd based on uuidvalue_igd */
+void complete_uuidvalues(void)
+{
+	size_t len;
+	len = strlen(uuidvalue_igd);
+	memcpy(uuidvalue_wan, uuidvalue_igd, len+1);
+	switch(uuidvalue_wan[len-1]) {
+	case '9':
+		uuidvalue_wan[len-1] = 'a';
+		break;
+	case 'f':
+		uuidvalue_wan[len-1] = '0';
+		break;
+	default:
+		uuidvalue_wan[len-1]++;
+	}
+	memcpy(uuidvalue_wcd, uuidvalue_wan, len+1);
+	switch(uuidvalue_wcd[len-1]) {
+	case '9':
+		uuidvalue_wcd[len-1] = 'a';
+		break;
+	case 'f':
+		uuidvalue_wcd[len-1] = '0';
+		break;
+	default:
+		uuidvalue_wcd[len-1]++;
+	}
+}
+
 /* init phase :
  * 1) read configuration file
  * 2) read command line arguments
@@ -651,12 +687,14 @@ init(int argc, char * * argv, struct runtime_vars * v)
 	int i;
 	int pid;
 	int debug_flag = 0;
-	int options_flag = 0;
 	int openlog_option;
 	struct sigaction sa;
 	/*const char * logfilename = 0;*/
 	const char * presurl = 0;
+#ifndef DISABLE_CONFIG_FILE
+	int options_flag = 0;
 	const char * optionsfile = DEFAULT_CONFIG;
+#endif /* DISABLE_CONFIG_FILE */
 	struct lan_addr_s * lan_addr;
 	struct lan_addr_s * lan_addr2;
 
@@ -666,6 +704,7 @@ init(int argc, char * * argv, struct runtime_vars * v)
 		if(0 == strcmp(argv[i], "-h"))
 			goto print_usage;
 	}
+#ifndef DISABLE_CONFIG_FILE
 	/* first check if "-f" option is used */
 	for(i=2; i<argc; i++)
 	{
@@ -676,16 +715,17 @@ init(int argc, char * * argv, struct runtime_vars * v)
 			break;
 		}
 	}
+#endif /* DISABLE_CONFIG_FILE */
 
 	/* set initial values */
-	SETFLAG(ENABLEUPNPMASK);
+	SETFLAG(ENABLEUPNPMASK);	/* UPnP is enabled by default */
 
 	LIST_INIT(&lan_addrs);
 	v->port = -1;
 	v->notify_interval = 30;	/* seconds between SSDP announces */
 	v->clean_ruleset_threshold = 20;
 	v->clean_ruleset_interval = 0;	/* interval between ruleset check. 0=disabled */
-
+#ifndef DISABLE_CONFIG_FILE
 	/* read options file first since
 	 * command line arguments have final say */
 	if(readoptionsfile(optionsfile) < 0)
@@ -752,13 +792,16 @@ init(int argc, char * * argv, struct runtime_vars * v)
 				if(strcmp(ary_options[i].value, "yes") == 0)
 					SETFLAG(SYSUPTIMEMASK);	/*sysuptime = 1;*/
 				break;
+#if defined(USE_PF) || defined(USE_IPF)
 			case UPNPPACKET_LOG:
 				if(strcmp(ary_options[i].value, "yes") == 0)
 					SETFLAG(LOGPACKETSMASK);	/*logpackets = 1;*/
 				break;
+#endif
 			case UPNPUUID:
-				strncpy(uuidvalue+5, ary_options[i].value,
-				        strlen(uuidvalue+5) + 1);
+				strncpy(uuidvalue_igd+5, ary_options[i].value,
+				        strlen(uuidvalue_igd+5) + 1);
+				complete_uuidvalues();
 				break;
 			case UPNPSERIAL:
 				strncpy(serialnumber, ary_options[i].value, SERIALNUMBER_MAX_LEN);
@@ -823,6 +866,7 @@ init(int argc, char * * argv, struct runtime_vars * v)
 			}
 		}
 	}
+#endif /* DISABLE_CONFIG_FILE */
 
 	/* command line arguments processing */
 	for(i=1; i<argc; i++)
@@ -845,12 +889,26 @@ init(int argc, char * * argv, struct runtime_vars * v)
 			else
 				fprintf(stderr, "Option -%c takes one argument.\n", argv[i][1]);
 			break;
-		case 'u':
+		case 'r':
 			if(i+1 < argc)
-				strncpy(uuidvalue+5, argv[++i], strlen(uuidvalue+5) + 1);
+				v->clean_ruleset_interval = atoi(argv[++i]);
 			else
 				fprintf(stderr, "Option -%c takes one argument.\n", argv[i][1]);
 			break;
+		case 'u':
+			if(i+1 < argc) {
+				strncpy(uuidvalue_igd+5, argv[++i], strlen(uuidvalue_igd+5) + 1);
+				complete_uuidvalues();
+			} else
+				fprintf(stderr, "Option -%c takes one argument.\n", argv[i][1]);
+			break;
+		case 'z':
+			if(i+1 < argc)
+				strncpy(friendly_name, argv[++i], FRIENDLY_NAME_MAX_LEN);
+			else
+				fprintf(stderr, "Option -%c takes one argument.\n", argv[i][1]);
+			friendly_name[FRIENDLY_NAME_MAX_LEN-1] = '\0';
+			break;
 		case 's':
 			if(i+1 < argc)
 				strncpy(serialnumber, argv[++i], SERIALNUMBER_MAX_LEN);
@@ -878,10 +936,12 @@ init(int argc, char * * argv, struct runtime_vars * v)
 		/*case 'l':
 			logfilename = argv[++i];
 			break;*/
+#if defined(USE_PF) || defined(USE_IPF)
 		case 'L':
 			/*logpackets = 1;*/
 			SETFLAG(LOGPACKETSMASK);
 			break;
+#endif
 		case 'S':
 			SETFLAG(SECUREMODEMASK);
 			break;
@@ -958,6 +1018,7 @@ init(int argc, char * * argv, struct runtime_vars * v)
 				fprintf(stderr, "Option -%c takes two arguments.\n", argv[i][1]);
 			break;
 		case 'a':
+#ifndef MULTIPLE_EXTERNAL_IP
 			if(i+1 < argc)
 			{
 				i++;
@@ -984,6 +1045,63 @@ init(int argc, char * * argv, struct runtime_vars * v)
 			}
 			else
 				fprintf(stderr, "Option -%c takes one argument.\n", argv[i][1]);
+#else
+			if(i+2 < argc)
+			{
+				char *val=calloc((strlen(argv[i+1]) + strlen(argv[i+2]) + 1), sizeof(char));
+				if (val == NULL)
+				{
+					fprintf(stderr, "memory allocation error for listen address storage\n");
+					break;
+				}
+				sprintf(val, "%s %s", argv[i+1], argv[i+2]);
+
+				lan_addr = (struct lan_addr_s *) malloc(sizeof(struct lan_addr_s));
+				if (lan_addr == NULL)
+				{
+					fprintf(stderr, "malloc(sizeof(struct lan_addr_s)): %m");
+					free(val);
+					break;
+				}
+				if(parselanaddr(lan_addr, val) != 0)
+				{
+					fprintf(stderr, "can't parse \"%s\" as valid lan address\n", val);
+					free(lan_addr);
+					free(val);
+					break;
+				}
+				/* check if we already have this address */
+				for(lan_addr2 = lan_addrs.lh_first; lan_addr2 != NULL; lan_addr2 = lan_addr2->list.le_next)
+				{
+					if (0 == strncmp(lan_addr2->str, lan_addr->str, 15))
+						break;
+				}
+				if (lan_addr2 == NULL)
+					LIST_INSERT_HEAD(&lan_addrs, lan_addr, list);
+
+				free(val);
+				i+=2;
+			}
+			else
+				fprintf(stderr, "Option -%c takes two arguments.\n", argv[i][1]);
+#endif
+			break;
+		case 'A':
+			if(i+1 < argc) {
+				void * tmp;
+				tmp = realloc(upnppermlist, sizeof(struct upnpperm) * (num_upnpperm+1));
+				if(tmp == NULL) {
+					fprintf(stderr, "memory allocation error for permission\n");
+				} else {
+					upnppermlist = tmp;
+					if(read_permission_line(upnppermlist + num_upnpperm, argv[++i]) >= 0) {
+						num_upnpperm++;
+					} else {
+						fprintf(stderr, "Permission rule parsing error :\n%s\n", argv[i]);
+					}
+				}
+			} else
+				fprintf(stderr, "Option -%c takes one argument.\n", argv[i][1]);
 			break;
 		case 'f':
 			i++;	/* discarding, the config file is already read */
@@ -1097,28 +1215,44 @@ init(int argc, char * * argv, struct runtime_vars * v)
 	return 0;
 print_usage:
 	fprintf(stderr, "Usage:\n\t"
-	        "%s [-f config_file] [-i ext_ifname] [-o ext_ip]\n"
-#ifndef ENABLE_NATPMP
-			"\t\t[-a listening_ip] [-p port] [-d] [-L] [-U] [-S]\n"
-#else
-			"\t\t[-a listening_ip] [-p port] [-d] [-L] [-U] [-S] [-N]\n"
+	        "%s "
+#ifndef DISABLE_CONFIG_FILE
+			"[-f config_file] "
 #endif
+			"[-i ext_ifname] [-o ext_ip]\n"
+#ifndef MULTIPLE_EXTERNAL_IP
+			"\t\t[-a listening_ip]"
+#else
+			"\t\t[-a listening_ip ext_ip]"
+#endif
+			" [-p port] [-d]"
+#if defined(USE_PF) || defined(USE_IPF)
+			" [-L]"
+#endif
+			" [-U] [-S]"
+#ifdef ENABLE_NATPMP
+			" [-N]"
+#endif
+			"\n"
 			/*"[-l logfile] " not functionnal */
 			"\t\t[-u uuid] [-s serial] [-m model_number] \n"
-			"\t\t[-t notify_interval] [-P pid_filename]\n"
-			"\t\t[-B down up] [-w url]\n"
+			"\t\t[-t notify_interval] [-P pid_filename] [-z fiendly_name]\n"
+			"\t\t[-B down up] [-w url] [-r clean_ruleset_interval]\n"
 #ifdef USE_PF
                         "\t\t[-q queue] [-T tag]\n"
 #endif
 #ifdef ENABLE_NFQUEUE
                         "\t\t[-Q queue] [-n name]\n"
 #endif
+			"\t\t[-A \"permission rule\"]\n"
 	        "\nNotes:\n\tThere can be one or several listening_ips.\n"
 	        "\tNotify interval is in seconds. Default is 30 seconds.\n"
 			"\tDefault pid file is '%s'.\n"
 			"\tDefault config file is '%s'.\n"
 			"\tWith -d miniupnpd will run as a standard program.\n"
+#if defined(USE_PF) || defined(USE_IPF)
 			"\t-L sets packet log in pf and ipf on.\n"
+#endif
 			"\t-S sets \"secure\" mode : clients can only add mappings to their own ip\n"
 			"\t-U causes miniupnpd to report system uptime instead "
 			"of daemon uptime.\n"
@@ -1132,9 +1266,14 @@ print_usage:
 			"\t-T sets the tag name in pf.\n"
 #endif
 #ifdef ENABLE_NFQUEUE
-                        "\t-Q sets the queue number that is used by NFQUEUE.\n"
-                        "\t-n sets the name of the interface(s) that packets will arrive on.\n"
+			"\t-Q sets the queue number that is used by NFQUEUE.\n"
+			"\t-n sets the name of the interface(s) that packets will arrive on.\n"
 #endif
+			"\t-A use following syntax for permission rules :\n"
+			"\t  (allow|deny) (external port range) ip/mask (internal port range)\n"
+			"\texamples :\n"
+			"\t  \"allow 1024-65535 192.168.1.0/24 1024-65535\"\n"
+			"\t  \"deny 0-65535 0.0.0.0/0 0-65535\"\n"
 			"\t-h prints this help and quits.\n"
 	        "", argv[0], pidfilename, DEFAULT_CONFIG);
 	return 1;
@@ -1221,6 +1360,15 @@ main(int argc, char * * argv)
 		return 0;
 	}
 
+	syslog(LOG_INFO, "Starting%s%swith external interface %s",
+#ifdef ENABLE_NATPMP
+	       GETFLAG(ENABLENATPMPMASK) ? " NAT-PMP " : " ",
+#else
+	       " ",
+#endif
+	       GETFLAG(ENABLEUPNPMASK) ? "UPnP-IGD " : "",
+	       ext_if_name);
+
 	if(GETFLAG(ENABLEUPNPMASK))
 	{
 
@@ -1255,7 +1403,7 @@ main(int argc, char * * argv)
 		sudp = OpenAndConfSSDPReceiveSocket(0);
 		if(sudp < 0)
 		{
-			syslog(LOG_INFO, "Failed to open socket for receiving SSDP. Trying to use MiniSSDPd");
+			syslog(LOG_NOTICE, "Failed to open socket for receiving SSDP. Trying to use MiniSSDPd");
 			if(SubmitServicesToMiniSSDPD(lan_addrs.lh_first->str, v.port) < 0) {
 				syslog(LOG_ERR, "Failed to connect to MiniSSDPd. EXITING");
 				return 1;
@@ -1265,7 +1413,7 @@ main(int argc, char * * argv)
 		sudpv6 = OpenAndConfSSDPReceiveSocket(1);
 		if(sudpv6 < 0)
 		{
-			syslog(LOG_INFO, "Failed to open socket for receiving SSDP (IP v6).");
+			syslog(LOG_WARNING, "Failed to open socket for receiving SSDP (IP v6).");
 		}
 #endif
 
@@ -1556,7 +1704,9 @@ main(int argc, char * * argv)
 				{
 					/*write(ectl->socket, buf, l);*/
 					write_command_line(ectl->socket, argc, argv);
+#ifndef DISABLE_CONFIG_FILE
 					write_option_list(ectl->socket);
+#endif
 					write_permlist(ectl->socket, upnppermlist, num_upnpperm);
 					write_upnphttp_details(ectl->socket, upnphttphead.lh_first);
 					write_ctlsockets_list(ectl->socket, ctllisthead.lh_first);
@@ -1587,13 +1737,21 @@ main(int argc, char * * argv)
 			struct sockaddr_un clientname;
 			struct ctlelem * tmp;
 			socklen_t clientnamelen = sizeof(struct sockaddr_un);
-			//syslog(LOG_DEBUG, "sctl!");
+			/*syslog(LOG_DEBUG, "sctl!");*/
 			s = accept(sctl, (struct sockaddr *)&clientname,
 			           &clientnamelen);
 			syslog(LOG_DEBUG, "sctl! : '%s'", clientname.sun_path);
 			tmp = malloc(sizeof(struct ctlelem));
-			tmp->socket = s;
-			LIST_INSERT_HEAD(&ctllisthead, tmp, entries);
+			if (tmp == NULL)
+			{
+				syslog(LOG_ERR, "Unable to allocate memory for ctlelem in main()");
+				close(s);
+			}
+			else
+			{
+				tmp->socket = s;
+				LIST_INSERT_HEAD(&ctllisthead, tmp, entries);
+			}
 		}
 #endif
 #ifdef ENABLE_EVENTS
@@ -1667,42 +1825,53 @@ main(int argc, char * * argv)
 
 				sockaddr_to_string((struct sockaddr *)&clientname, addr_str, sizeof(addr_str));
 				syslog(LOG_INFO, "HTTP connection from %s", addr_str);
-				/* Create a new upnphttp object and add it to
-				 * the active upnphttp object list */
-				tmp = New_upnphttp(shttp);
-				if(tmp)
+				if(get_lan_for_peer((struct sockaddr *)&clientname) == NULL)
 				{
-#ifdef ENABLE_IPV6
-					if(clientname.ss_family == AF_INET)
-					{
-						tmp->clientaddr = ((struct sockaddr_in *)&clientname)->sin_addr;
-					}
-					else if(clientname.ss_family == AF_INET6)
-					{
-						struct sockaddr_in6 * addr = (struct sockaddr_in6 *)&clientname;
-						if(IN6_IS_ADDR_V4MAPPED(&addr->sin6_addr))
-						{
-							memcpy(&tmp->clientaddr,
-							       &addr->sin6_addr.s6_addr[12],
-							       4);
-						}
-						else
-						{
-							tmp->ipv6 = 1;
-							memcpy(&tmp->clientaddr_v6,
-							       &addr->sin6_addr,
-							       sizeof(struct in6_addr));
-						}
-					}
-#else
-					tmp->clientaddr = clientname.sin_addr;
-#endif
-					LIST_INSERT_HEAD(&upnphttphead, tmp, entries);
+					/* The peer is not a LAN ! */
+					syslog(LOG_WARNING,
+					       "HTTP peer %s is not from a LAN, closing the connection",
+					       addr_str);
+					close(shttp);
 				}
 				else
 				{
-					syslog(LOG_ERR, "New_upnphttp() failed");
-					close(shttp);
+					/* Create a new upnphttp object and add it to
+					 * the active upnphttp object list */
+					tmp = New_upnphttp(shttp);
+					if(tmp)
+					{
+#ifdef ENABLE_IPV6
+						if(clientname.ss_family == AF_INET)
+						{
+							tmp->clientaddr = ((struct sockaddr_in *)&clientname)->sin_addr;
+						}
+						else if(clientname.ss_family == AF_INET6)
+						{
+							struct sockaddr_in6 * addr = (struct sockaddr_in6 *)&clientname;
+							if(IN6_IS_ADDR_V4MAPPED(&addr->sin6_addr))
+							{
+								memcpy(&tmp->clientaddr,
+								       &addr->sin6_addr.s6_addr[12],
+								       4);
+							}
+							else
+							{
+								tmp->ipv6 = 1;
+								memcpy(&tmp->clientaddr_v6,
+								       &addr->sin6_addr,
+								       sizeof(struct in6_addr));
+							}
+						}
+#else
+						tmp->clientaddr = clientname.sin_addr;
+#endif
+						LIST_INSERT_HEAD(&upnphttphead, tmp, entries);
+					}
+					else
+					{
+						syslog(LOG_ERR, "New_upnphttp() failed");
+						close(shttp);
+					}
 				}
 			}
 		}
@@ -1802,7 +1971,9 @@ shutdown:
 #endif
 	free(snotify);
 	closelog();
+#ifndef DISABLE_CONFIG_FILE
 	freeoptions();
+#endif
 
 	return 0;
 }
