@@ -1,10 +1,10 @@
-/* $Id: natpmp.h,v 1.8 2011/05/27 21:36:22 nanard Exp $ */
+/* $Id: natpmp.h,v 1.9 2012/09/27 15:47:15 nanard Exp $ */
 /* MiniUPnP project
  * author : Thomas Bernard
  * website : http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  */
-#ifndef __NATPMP_H__
-#define __NATPMP_H__
+#ifndef NATPMP_H_INCLUDED
+#define NATPMP_H_INCLUDED
 
 /* The NAT-PMP specification which can be found at the url :
  * http://files.dns-sd.org/draft-cheshire-nat-pmp.txt
