@@ -1,4 +1,4 @@
-/* $Id: options.h,v 1.20 2012/04/30 13:38:21 nanard Exp $ */
+/* $Id: options.h,v 1.22 2012/09/27 15:47:15 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * author: Ryan Wagoner
@@ -6,11 +6,12 @@
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
-#ifndef __OPTIONS_H__
-#define __OPTIONS_H__
+#ifndef OPTIONS_H_INCLUDED
+#define OPTIONS_H_INCLUDED
 
 #include "config.h"
 
+#ifndef DISABLE_CONFIG_FILE
 /* enum of option available in the miniupnpd.conf */
 enum upnpconfigoptions {
 	UPNP_INVALID = 0,
@@ -71,5 +72,7 @@ struct option
 extern struct option * ary_options;
 extern unsigned int num_options;
 
-#endif
+#endif /* DISABLE_CONFIG_FILE */
+
+#endif /* OPTIONS_H_INCLUDED */
 
