@@ -1,7 +1,7 @@
-/* $Id: obsdrdr.c,v 1.71 2012/03/11 08:46:19 nanard Exp $ */
+/* $Id: obsdrdr.c,v 1.74 2012/05/01 09:20:43 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2010 Thomas Bernard
+ * (c) 2006-2012 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -55,6 +55,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "../macros.h"
 #include "../config.h"
 #include "obsdrdr.h"
 #include "../upnpglobalvars.h"
@@ -102,7 +103,7 @@ remove_timestamp_entry(unsigned short eport, int proto)
 }
 
 /* /dev/pf when opened */
-static int dev = -1;
+int dev = -1;
 
 /* shutdown_redirect() :
  * close the /dev/pf device */
@@ -327,6 +328,10 @@ add_filter_rule2(const char * ifname,
 				 int proto, const char * desc)
 {
 #ifndef PF_ENABLE_FILTER_RULES
+	UNUSED(ifname);
+	UNUSED(rhost); UNUSED(iaddr);
+	UNUSED(eport); UNUSED(iport);
+	UNUSED(proto); UNUSED(desc);
 	return 0;
 #else
 	int r;
@@ -460,6 +465,8 @@ get_redirect_rule(const char * ifname, unsigned short eport, int proto,
 #ifndef PF_NEWSTYLE
 	struct pfioc_pooladdr pp;
 #endif
+	UNUSED(ifname);
+
 	if(dev<0) {
 		syslog(LOG_ERR, "pf device is not open");
 		return -1;
@@ -559,6 +566,8 @@ delete_redirect_rule(const char * ifname, unsigned short eport, int proto)
 {
 	int i, n;
 	struct pfioc_rule pr;
+	UNUSED(ifname);
+
 	if(dev<0) {
 		syslog(LOG_ERR, "pf device is not open");
 		return -1;
@@ -611,6 +620,7 @@ int
 delete_filter_rule(const char * ifname, unsigned short eport, int proto)
 {
 #ifndef PF_ENABLE_FILTER_RULES
+	UNUSED(ifname); UNUSED(eport); UNUSED(proto);
 	return 0;
 #else
 	int i, n;
@@ -868,7 +878,7 @@ list_rules(void)
 			perror("DIOCGETRULE");
 		printf(" %s %s %d:%d -> %d:%d  proto %d keep_state=%d action=%d\n",
 			pr.rule.ifname,
-			inet_ntop(AF_INET, &pr.rule.src.addr.v.a.addr.v4.s_addr, buf, 32);
+			inet_ntop(AF_INET, &pr.rule.src.addr.v.a.addr.v4.s_addr, buf, 32),
 			(int)ntohs(pr.rule.dst.port[0]),
 			(int)ntohs(pr.rule.dst.port[1]),
 #ifndef PF_NEWSTYLE
