@@ -1,12 +1,12 @@
-/* $Id: pfpinhole.h,v 1.9 2012/05/01 22:37:53 nanard Exp $ */
+/* $Id: pfpinhole.h,v 1.10 2012/09/27 15:44:10 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2012 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
-#ifndef __PFPINHOLE_H__
-#define __PFPINHOLE_H__
+#ifndef PFPINHOLE_H_INCLUDED
+#define PFPINHOLE_H_INCLUDED
 
 #ifdef ENABLE_6FC_SERVICE
 int add_pinhole(const char * ifname,
