@@ -1,7 +1,7 @@
-/* $Id: upnpdescgen.c,v 1.67 2012/04/30 14:03:52 nanard Exp $ */
+/* $Id: upnpdescgen.c,v 1.75 2013/07/30 06:55:19 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2012 Thomas Bernard
+ * (c) 2006-2013 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -154,7 +154,7 @@ static const struct XMLElt rootDesc[] =
 	{"/modelNumber", modelnumber},
 	{"/modelURL", ROOTDEV_MODELURL},
 	{"/serialNumber", serialnumber},
-	{"/UDN", uuidvalue},	/* required */
+	{"/UDN", uuidvalue_igd},	/* required */
 	/* see if /UPC is needed. */
 #ifdef ENABLE_6FC_SERVICE
 #define SERVICES_OFFSET 63
@@ -201,8 +201,8 @@ static const struct XMLElt rootDesc[] =
 	{"/modelNumber", WANDEV_MODELNUMBER},
 	{"/modelURL", WANDEV_MODELURL},
 	{"/serialNumber", serialnumber},
-	{"/UDN", uuidvalue},
-	{"/UPC", WANDEV_UPC},
+	{"/UDN", uuidvalue_wan},
+	{"/UPC", WANDEV_UPC},	/* UPC (=12 digit barcode) is optional */
 /* 30 */
 	{"serviceList", INITHELPER(32,1)},
 	{"deviceList", INITHELPER(38,1)},
@@ -229,8 +229,8 @@ static const struct XMLElt rootDesc[] =
 	{"/modelNumber", WANCDEV_MODELNUMBER},
 	{"/modelURL", WANCDEV_MODELURL},
 	{"/serialNumber", serialnumber},
-	{"/UDN", uuidvalue},
-	{"/UPC", WANCDEV_UPC},
+	{"/UDN", uuidvalue_wcd},
+	{"/UPC", WANCDEV_UPC},	/* UPC (=12 digit Barcode) is optional */
 #ifdef ENABLE_6FC_SERVICE
 	{"serviceList", INITHELPER(51,2)},
 #else
@@ -423,15 +423,15 @@ static const struct action WANIPCnActions[] =
 	{"SetConnectionType", SetConnectionTypeArgs}, /* R */
 	{"GetConnectionTypeInfo", GetConnectionTypeInfoArgs}, /* R */
 	{"RequestConnection", 0}, /* R */
-	{"RequestTermination", 0}, /* O */
+	/*{"RequestTermination", 0},*/ /* O */
 	{"ForceTermination", 0}, /* R */
 	/*{"SetAutoDisconnectTime", 0},*/ /* O */
 	/*{"SetIdleDisconnectTime", 0},*/ /* O */
 	/*{"SetWarnDisconnectDelay", 0}, */ /* O */
 	{"GetStatusInfo", GetStatusInfoArgs}, /* R */
-	/*GetAutoDisconnectTime*/
-	/*GetIdleDisconnectTime*/
-	/*GetWarnDisconnectDelay*/
+	/*GetAutoDisconnectTime*/ /* O */
+	/*GetIdleDisconnectTime*/ /* O */
+	/*GetWarnDisconnectDelay*/ /* O */
 	{"GetNATRSIPStatus", GetNATRSIPStatusArgs}, /* R */
 	{"GetGenericPortMappingEntry", GetGenericPortMappingEntryArgs}, /* R */
 	{"GetSpecificPortMappingEntry", GetSpecificPortMappingEntryArgs}, /* R */
@@ -466,6 +466,9 @@ static const struct action WANIPCnActions[] =
 };
 /* R=Required, O=Optional */
 
+/* ignore "warning: missing initializer" */
+#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
+
 static const struct stateVar WANIPCnVars[] =
 {
 /* 0 */
@@ -1131,7 +1134,7 @@ genDP(int * len)
 
 #ifdef ENABLE_EVENTS
 static char *
-genEventVars(int * len, const struct serviceDesc * s, const char * servns)
+genEventVars(int * len, const struct serviceDesc * s)
 {
 	char tmp[16];
 	const struct stateVar * v;
@@ -1143,15 +1146,13 @@ genEventVars(int * len, const struct serviceDesc * s, const char * servns)
 		return NULL;
 	*len = 0;
 	v = s->serviceStateTable;
-	str = strcat_str(str, len, &tmplen, "<e:propertyset xmlns:e=\"urn:schemas-upnp-org:event-1-0\" xmlns:s=\"");
-	str = strcat_str(str, len, &tmplen, servns);
-	str = strcat_str(str, len, &tmplen, "\">");
+	str = strcat_str(str, len, &tmplen, "<e:propertyset xmlns:e=\"urn:schemas-upnp-org:event-1-0\">");
 	while(v->name) {
 		if(v->itype & 0x80) {
-			str = strcat_str(str, len, &tmplen, "<e:property><s:");
+			str = strcat_str(str, len, &tmplen, "<e:property><");
 			str = strcat_str(str, len, &tmplen, v->name);
 			str = strcat_str(str, len, &tmplen, ">");
-			/*printf("<e:property><s:%s>", v->name);*/
+			/*printf("<e:property><%s>", v->name);*/
 			switch(v->ieventvalue) {
 			case 0:
 				break;
@@ -1195,7 +1196,7 @@ genEventVars(int * len, const struct serviceDesc * s, const char * servns)
 					str = strcat_str(str, len, &tmplen, use_ext_ip_addr);
 				else {
 					char ext_ip_addr[INET_ADDRSTRLEN];
-					if(getifaddr(ext_if_name, ext_ip_addr, INET_ADDRSTRLEN) < 0) {
+					if(getifaddr(ext_if_name, ext_ip_addr, INET_ADDRSTRLEN, NULL, NULL) < 0) {
 						str = strcat_str(str, len, &tmplen, "0.0.0.0");
 					} else {
 						str = strcat_str(str, len, &tmplen, ext_ip_addr);
@@ -1204,16 +1205,20 @@ genEventVars(int * len, const struct serviceDesc * s, const char * servns)
 				break;
 			case DEFAULTCONNECTIONSERVICE_MAGICALVALUE:
 				/* DefaultConnectionService magical value */
-				str = strcat_str(str, len, &tmplen, uuidvalue);
+				str = strcat_str(str, len, &tmplen, uuidvalue_wcd);
+#ifdef IGD_V2
+				str = strcat_str(str, len, &tmplen, ":WANConnectionDevice:2,urn:upnp-org:serviceId:WANIPConn1");
+#else
 				str = strcat_str(str, len, &tmplen, ":WANConnectionDevice:1,urn:upnp-org:serviceId:WANIPConn1");
+#endif
 				break;
 			default:
 				str = strcat_str(str, len, &tmplen, upnpallowedvalues[v->ieventvalue]);
 			}
-			str = strcat_str(str, len, &tmplen, "</s:");
+			str = strcat_str(str, len, &tmplen, "</");
 			str = strcat_str(str, len, &tmplen, v->name);
 			str = strcat_str(str, len, &tmplen, "></e:property>");
-			/*printf("</s:%s></e:property>\n", v->name);*/
+			/*printf("</%s></e:property>\n", v->name);*/
 		}
 		v++;
 	}
@@ -1231,16 +1236,14 @@ char *
 getVarsWANIPCn(int * l)
 {
 	return genEventVars(l,
-                        &scpdWANIPCn,
-	                    SERVICE_TYPE_WANIPC);
+                        &scpdWANIPCn);
 }
 
 char *
 getVarsWANCfg(int * l)
 {
 	return genEventVars(l,
-	                    &scpdWANCfg,
-	                    "urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1");
+	                    &scpdWANCfg);
 }
 
 #ifdef ENABLE_L3F_SERVICE
@@ -1248,8 +1251,7 @@ char *
 getVarsL3F(int * l)
 {
 	return genEventVars(l,
-	                    &scpdL3F,
-	                    "urn:schemas-upnp-org:service:Layer3Forwarding:1");
+	                    &scpdL3F);
 }
 #endif
 
@@ -1258,8 +1260,7 @@ char *
 getVars6FC(int * l)
 {
 	return genEventVars(l,
-	                    &scpd6FC,
-	                    "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1");
+	                    &scpd6FC);
 }
 #endif
 
@@ -1268,8 +1269,7 @@ char *
 getVarsDP(int * l)
 {
 	return genEventVars(l,
-	                    &scpdDP,
-	                    "urn:schemas-upnp-org:service:DeviceProtection:1");
+	                    &scpdDP);
 }
 #endif
 
