@@ -1,7 +1,7 @@
-/* $Id: upnpevents.c,v 1.24 2012/04/30 21:21:33 nanard Exp $ */
+/* $Id: upnpevents.c,v 1.27 2013/06/13 13:21:30 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2008-2012 Thomas Bernard
+ * (c) 2008-2013 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -93,6 +93,14 @@ newSubscriber(const char * eventurl, const char * callback, int callbacklen)
 #ifdef ENABLE_L3F_SERVICE
 	else if(strcmp(eventurl, L3F_EVENTURL)==0)
 		tmp->service = EL3F;
+#endif
+#ifdef ENABLE_6FC_SERVICE
+	else if(strcmp(eventurl, WANIP6FC_EVENTURL)==0)
+		tmp->service = E6FC;
+#endif
+#ifdef ENABLE_DP_SERVICE
+	else if(strcmp(eventurl, DP_EVENTURL)==0)
+		tmp->service = EDP;
 #endif
 	else {
 		free(tmp);
@@ -102,7 +110,7 @@ newSubscriber(const char * eventurl, const char * callback, int callbacklen)
 	tmp->callback[callbacklen] = '\0';
 	/* make a dummy uuid */
 	/* TODO: improve that */
-	strncpy(tmp->uuid, uuidvalue, sizeof(tmp->uuid));
+	strncpy(tmp->uuid, uuidvalue_igd, sizeof(tmp->uuid));
 	tmp->uuid[sizeof(tmp->uuid)-1] = '\0';
 	snprintf(tmp->uuid+37, 5, "%04lx", random() & 0xffff);
 	return tmp;
@@ -418,9 +426,23 @@ static void upnp_event_recv(struct upnp_event_notify * obj)
 static void
 upnp_event_process_notify(struct upnp_event_notify * obj)
 {
+	int err;
+	socklen_t len;
 	switch(obj->state) {
 	case EConnecting:
 		/* now connected or failed to connect */
+		len = sizeof(err);
+		if(getsockopt(obj->s, SOL_SOCKET, SO_ERROR, &err, &len) < 0) {
+			syslog(LOG_ERR, "%s: getsockopt: %m", "upnp_event_process_notify");
+			obj->state = EError;
+			break;
+		}
+		if(err != 0) {
+			errno = err;
+			syslog(LOG_WARNING, "%s: connect failed: %m", "upnp_event_process_notify");
+			obj->state = EError;
+			break;
+		}
 		upnp_event_prepare(obj);
 		if(obj->state == ESending)
 			upnp_event_send(obj);
@@ -436,7 +458,7 @@ upnp_event_process_notify(struct upnp_event_notify * obj)
 		obj->s = -1;
 		break;
 	default:
-		syslog(LOG_ERR, "upnp_event_process_notify: unknown state");
+		syslog(LOG_ERR, "%s: unknown state", "upnp_event_process_notify");
 	}
 }
 
