@@ -1,12 +1,12 @@
-/* $Id: upnpglobalvars.h,v 1.33 2012/04/26 14:01:16 nanard Exp $ */
+/* $Id: upnpglobalvars.h,v 1.35 2013/06/13 13:21:30 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006-2012 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
-#ifndef __UPNPGLOBALVARS_H__
-#define __UPNPGLOBALVARS_H__
+#ifndef UPNPGLOBALVARS_H_INCLUDED
+#define UPNPGLOBALVARS_H_INCLUDED
 
 #include <time.h>
 #include "upnppermissions.h"
@@ -54,7 +54,9 @@ extern int runtime_flags;
 
 extern const char * pidfilename;
 
-extern char uuidvalue[];
+extern char uuidvalue_igd[];	/* uuid of root device (IGD) */
+extern char uuidvalue_wan[];	/* uuid of WAN Device */
+extern char uuidvalue_wcd[];	/* uuid of WAN Connection Device */
 
 #define SERIALNUMBER_MAX_LEN (10)
 extern char serialnumber[];
