@@ -1,4 +1,4 @@
-/* $Id: upnphttp.c,v 1.71 2012/04/30 13:46:29 nanard Exp $ */
+/* $Id: upnphttp.c,v 1.86 2013/02/07 10:26:07 nanard Exp $ */
 /* Project :  miniupnp
  * Website :  http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * Author :   Thomas Bernard
@@ -19,6 +19,9 @@
 #include <ctype.h>
 #include <errno.h>
 #include "config.h"
+#ifdef ENABLE_HTTP_DATE
+#include <time.h>
+#endif
 #include "upnphttp.h"
 #include "upnpdescgen.h"
 #include "miniupnpdpath.h"
@@ -68,7 +71,9 @@ Delete_upnphttp(struct upnphttp * h)
 	}
 }
 
-/* parse HttpHeaders of the REQUEST */
+/* parse HttpHeaders of the REQUEST
+ * This function is called after the \r\n\r\n character
+ * sequence has been found in h->req_buf */
 static void
 ParseHttpHeaders(struct upnphttp * h)
 {
@@ -76,11 +81,21 @@ ParseHttpHeaders(struct upnphttp * h)
 	char * colon;
 	char * p;
 	int n;
+	if((h->req_buf == NULL) || (h->req_contentoff <= 0))
+		return;
 	line = h->req_buf;
-	/* TODO : check if req_buf, contentoff are ok */
 	while(line < (h->req_buf + h->req_contentoff))
 	{
-		colon = strchr(line, ':');
+		colon = line;
+		while(*colon != ':')
+		{
+			if(*colon == '\r' || *colon == '\n')
+			{
+				colon = NULL;	/* no ':' character found on the line */
+				break;
+			}
+			colon++;
+		}
 		if(colon)
 		{
 			if(strncasecmp(line, "Content-Length", 14)==0)
@@ -89,6 +104,10 @@ ParseHttpHeaders(struct upnphttp * h)
 				while(*p < '0' || *p > '9')
 					p++;
 				h->req_contentlen = atoi(p);
+				if(h->req_contentlen < 0) {
+					syslog(LOG_WARNING, "ParseHttpHeaders() invalid Content-Length %d", h->req_contentlen);
+					h->req_contentlen = 0;
+				}
 				/*printf("*** Content-Lenght = %d ***\n", h->req_contentlen);
 				printf("    readbufflen=%d contentoff = %d\n",
 					h->req_buflen, h->req_contentoff);*/
@@ -100,9 +119,7 @@ ParseHttpHeaders(struct upnphttp * h)
 				while(*p == ':' || *p == ' ' || *p == '\t')
 					p++;
 				while(p[n]>=' ')
-				{
 					n++;
-				}
 				if((p[0] == '"' && p[n-1] == '"')
 				  || (p[0] == '\'' && p[n-1] == '\''))
 				{
@@ -111,6 +128,37 @@ ParseHttpHeaders(struct upnphttp * h)
 				h->req_soapActionOff = p - h->req_buf;
 				h->req_soapActionLen = n;
 			}
+			else if(strncasecmp(line, "accept-language", 15) == 0)
+			{
+				p = colon;
+				n = 0;
+				while(*p == ':' || *p == ' ' || *p == '\t')
+					p++;
+				while(p[n]>=' ')
+					n++;
+				syslog(LOG_DEBUG, "accept-language HTTP header : '%.*s'", n, p);
+				/* keep only the 1st accepted language */
+				n = 0;
+				while(p[n]>' ' && p[n] != ',')
+					n++;
+				if(n >= (int)sizeof(h->accept_language))
+					n = (int)sizeof(h->accept_language) - 1;
+				memcpy(h->accept_language, p, n);
+				h->accept_language[n] = '\0';
+			}
+			else if(strncasecmp(line, "expect", 6) == 0)
+			{
+				p = colon;
+				n = 0;
+				while(*p == ':' || *p == ' ' || *p == '\t')
+					p++;
+				while(p[n]>=' ')
+					n++;
+				if(strncasecmp(p, "100-continue", 12) == 0) {
+					h->respflags |= FLAG_CONTINUE;
+					syslog(LOG_DEBUG, "\"Expect: 100-Continue\" header detected");
+				}
+			}
 #ifdef ENABLE_EVENTS
 			else if(strncasecmp(line, "Callback", 8)==0)
 			{
@@ -150,8 +198,24 @@ intervening space) by either an integer or the keyword "infinite". */
 					h->req_Timeout = atoi(p+7);
 				}
 			}
+#ifdef UPNP_STRICT
+			else if(strncasecmp(line, "nt", 2)==0)
+			{
+				p = colon + 1;
+				while(isspace(*p))
+					p++;
+				n = 0;
+				while(!isspace(p[n]))
+					n++;
+				h->req_NTOff = p - h->req_buf;
+				h->req_NTLen = n;
+			}
+#endif
 #endif
 		}
+		/* the loop below won't run off the end of the buffer
+		 * because the buffer is guaranteed to contain the \r\n\r\n
+		 * character sequence */
 		while(!(line[0] == '\r' && line[1] == '\n'))
 			line++;
 		line += 2;
@@ -173,6 +237,20 @@ Send404(struct upnphttp * h)
 	SendRespAndClose_upnphttp(h);
 }
 
+static void
+Send405(struct upnphttp * h)
+{
+	static const char body405[] =
+		"<HTML><HEAD><TITLE>405 Method Not Allowed</TITLE></HEAD>"
+		"<BODY><H1>Method Not Allowed</H1>The HTTP Method "
+		"is not allowed on this resource.</BODY></HTML>\r\n";
+
+	h->respflags |= FLAG_HTML;
+	BuildResp2_upnphttp(h, 405, "Method Not Allowed",
+	                    body405, sizeof(body405) - 1);
+	SendRespAndClose_upnphttp(h);
+}
+
 /* very minimalistic 501 error message */
 static void
 Send501(struct upnphttp * h)
@@ -188,10 +266,13 @@ Send501(struct upnphttp * h)
 	SendRespAndClose_upnphttp(h);
 }
 
+/* findendheaders() find the \r\n\r\n character sequence and
+ * return a pointer to it.
+ * It returns NULL if not found */
 static const char *
 findendheaders(const char * s, int len)
 {
-	while(len-->0)
+	while(len-->3)
 	{
 		if(s[0]=='\r' && s[1]=='\n' && s[2]=='\r' && s[3]=='\n')
 			return s;
@@ -249,6 +330,7 @@ ProcessHTTPPOST_upnphttp(struct upnphttp * h)
 {
 	if((h->req_buflen - h->req_contentoff) >= h->req_contentlen)
 	{
+		/* the request body is received */
 		if(h->req_soapActionOff > 0)
 		{
 			/* we can process the request */
@@ -269,6 +351,20 @@ ProcessHTTPPOST_upnphttp(struct upnphttp * h)
 			SendRespAndClose_upnphttp(h);
 		}
 	}
+	else if(h->respflags & FLAG_CONTINUE)
+	{
+		/* Sending the 100 Continue response */
+		if(!h->res_buf) {
+			h->res_buf = malloc(256);
+			h->res_buf_alloclen = 256;
+		}
+		h->res_buflen = snprintf(h->res_buf, h->res_buf_alloclen,
+		                         "%s 100 Continue\r\n\r\n", h->HttpVer);
+		h->res_sent = 0;
+		h->state = ESendingContinue;
+		if(SendResp_upnphttp(h))
+			h->state = EWaitingForHttpContent;
+	}
 	else
 	{
 		/* waiting for remaining data */
@@ -364,6 +460,19 @@ ProcessHTTPSubscribe_upnphttp(struct upnphttp * h, const char * path)
 	/* Check that the callback URL is on the same IP as
 	 * the request, and not on the internet, nor on ourself (DOS attack ?) */
 		if(h->req_CallbackOff > 0) {
+#ifdef UPNP_STRICT
+			/* SID: and Callback: are incompatible */
+			if(h->req_SIDOff > 0) {
+				syslog(LOG_WARNING, "Both Callback: and SID: in SUBSCRIBE");
+				BuildResp2_upnphttp(h, 400, "Incompatible header fields", 0, 0);
+			/* "NT: upnp:event" header is mandatory */
+			} else if(h->req_NTOff <= 0 || h->req_NTLen != 10 ||
+			   0 != memcmp("upnp:event", h->req_buf + h->req_NTOff, 10)) {
+				syslog(LOG_WARNING, "Invalid NT in SUBSCRIBE %.*s",
+				       h->req_NTLen, h->req_buf + h->req_NTOff);
+				BuildResp2_upnphttp(h, 412, "Precondition Failed", 0, 0);
+			} else
+#endif
 			if(checkCallbackURL(h)) {
 				sid = upnpevents_addSubscriber(path, h->req_buf + h->req_CallbackOff,
 				                               h->req_CallbackLen, h->req_Timeout);
@@ -385,6 +494,13 @@ ProcessHTTPSubscribe_upnphttp(struct upnphttp * h, const char * path)
 412 Precondition Failed. If a SID does not correspond to a known,
 un-expired subscription, the publisher must respond
 with HTTP error 412 Precondition Failed. */
+#ifdef UPNP_STRICT
+			/* SID: and NT: headers are incompatibles */
+			if(h->req_NTOff > 0) {
+				syslog(LOG_WARNING, "Both NT: and SID: in SUBSCRIBE");
+				BuildResp2_upnphttp(h, 400, "Incompatible header fields", 0, 0);
+			} else
+#endif
 			if(renewSubscription(h->req_buf + h->req_SIDOff, h->req_SIDLen,
 			                     h->req_Timeout) < 0) {
 				BuildResp2_upnphttp(h, 412, "Precondition Failed", 0, 0);
@@ -403,6 +519,15 @@ ProcessHTTPUnSubscribe_upnphttp(struct upnphttp * h, const char * path)
 	syslog(LOG_DEBUG, "ProcessHTTPUnSubscribe %s", path);
 	syslog(LOG_DEBUG, "SID '%.*s'", h->req_SIDLen, h->req_buf + h->req_SIDOff);
 	/* Remove from the list */
+#ifdef UPNP_STRICT
+	if(h->req_SIDOff <= 0 || h->req_SIDLen == 0) {
+		/* SID: header missing or empty */
+		BuildResp2_upnphttp(h, 412, "Precondition Failed", 0, 0);
+	} else if(h->req_CallbackOff > 0 || h->req_NTOff > 0) {
+		/* no NT: or Callback: header must be present */
+		BuildResp2_upnphttp(h, 400, "Incompatible header fields", 0, 0);
+	} else
+#endif
 	if(upnpevents_removeSubscriber(h->req_buf + h->req_SIDOff, h->req_SIDLen) < 0) {
 		BuildResp2_upnphttp(h, 412, "Precondition Failed", 0, 0);
 	} else {
@@ -413,10 +538,33 @@ ProcessHTTPUnSubscribe_upnphttp(struct upnphttp * h, const char * path)
 #endif
 
 /* Parse and process Http Query
- * called once all the HTTP headers have been received. */
+ * called once all the HTTP headers have been received,
+ * so it is guaranteed that h->req_buf contains the \r\n\r\n
+ * character sequence */
 static void
 ProcessHttpQuery_upnphttp(struct upnphttp * h)
 {
+	static const struct {
+		const char * path;
+		char * (* f)(int *);
+	} path_desc[] = {
+		{ ROOTDESC_PATH, genRootDesc},
+		{ WANIPC_PATH, genWANIPCn},
+		{ WANCFG_PATH, genWANCfg},
+#ifdef HAS_DUMMY_SERVICE
+		{ DUMMY_PATH, NULL},
+#endif
+#ifdef ENABLE_L3F_SERVICE
+		{ L3F_PATH, genL3F},
+#endif
+#ifdef ENABLE_6FC_SERVICE
+		{ WANIP6FC_PATH, gen6FC},
+#endif
+#ifdef ENABLE_DP_SERVICE
+		{ DP_PATH, genDP},
+#endif
+		{ NULL, NULL}
+	};
 	char HttpCommand[16];
 	char HttpUrl[128];
 	char * HttpVer;
@@ -425,6 +573,9 @@ ProcessHttpQuery_upnphttp(struct upnphttp * h)
 	p = h->req_buf;
 	if(!p)
 		return;
+	/* note : checking (*p != '\r') is enough to avoid runing off the
+	 * end of the buffer, because h->req_buf is guaranteed to contain
+	 * the \r\n\r\n character sequence */
 	for(i = 0; i<15 && *p != ' ' && *p != '\r'; i++)
 		HttpCommand[i] = *(p++);
 	HttpCommand[i] = '\0';
@@ -450,47 +601,37 @@ ProcessHttpQuery_upnphttp(struct upnphttp * h)
 	else if(strcmp("GET", HttpCommand) == 0)
 	{
 		h->req_command = EGet;
-		if(strcasecmp(ROOTDESC_PATH, HttpUrl) == 0)
-		{
-			sendXMLdesc(h, genRootDesc);
-		}
-		else if(strcasecmp(WANIPC_PATH, HttpUrl) == 0)
-		{
-			sendXMLdesc(h, genWANIPCn);
-		}
-		else if(strcasecmp(WANCFG_PATH, HttpUrl) == 0)
-		{
-			sendXMLdesc(h, genWANCfg);
-		}
+		for(i=0; path_desc[i].path; i++) {
+			if(strcasecmp(path_desc[i].path, HttpUrl) == 0) {
+				if(path_desc[i].f)
+					sendXMLdesc(h, path_desc[i].f);
+				else
 #ifdef HAS_DUMMY_SERVICE
-		else if(strcasecmp(DUMMY_PATH, HttpUrl) == 0)
-		{
-			sendDummyDesc(h);
+					sendDummyDesc(h);
+#else
+					continue;
+#endif
+				return;
+			}
+		}
+		if(0 == memcmp(HttpUrl, "/ctl/", 5)) {
+			/* 405 Method Not Allowed
+			 * Allow: POST */
+			h->respflags = FLAG_ALLOW_POST;
+			Send405(h);
+			return;
+		}
+#ifdef ENABLE_EVENTS
+		if(0 == memcmp(HttpUrl, "/evt/", 5)) {
+			/* 405 Method Not Allowed
+			 * Allow: SUBSCRIBE, UNSUBSCRIBE */
+			h->respflags = FLAG_ALLOW_SUB_UNSUB;
+			Send405(h);
+			return;
 		}
 #endif
-#ifdef ENABLE_L3F_SERVICE
-		else if(strcasecmp(L3F_PATH, HttpUrl) == 0)
-		{
-			sendXMLdesc(h, genL3F);
-		}
-#endif
-#ifdef ENABLE_6FC_SERVICE
-		else if(strcasecmp(WANIP6FC_PATH, HttpUrl) == 0)
-		{
-			sendXMLdesc(h, gen6FC);
-		}
-#endif
-#ifdef ENABLE_DP_SERVICE
-		else if(strcasecmp(DP_PATH, HttpUrl) == 0)
-		{
-			sendXMLdesc(h, genDP);
-		}
-#endif
-		else
-		{
-			syslog(LOG_NOTICE, "%s not found, responding ERROR 404", HttpUrl);
-			Send404(h);
-		}
+		syslog(LOG_NOTICE, "%s not found, responding ERROR 404", HttpUrl);
+		Send404(h);
 	}
 #ifdef ENABLE_EVENTS
 	else if(strcmp("SUBSCRIBE", HttpCommand) == 0)
@@ -521,6 +662,7 @@ ProcessHttpQuery_upnphttp(struct upnphttp * h)
 void
 Process_upnphttp(struct upnphttp * h)
 {
+	char * h_tmp;
 	char buf[2048];
 	int n;
 
@@ -551,14 +693,25 @@ Process_upnphttp(struct upnphttp * h)
 			const char * endheaders;
 			/* if 1st arg of realloc() is null,
 			 * realloc behaves the same as malloc() */
-			h->req_buf = (char *)realloc(h->req_buf, n + h->req_buflen + 1);
-			memcpy(h->req_buf + h->req_buflen, buf, n);
-			h->req_buflen += n;
-			h->req_buf[h->req_buflen] = '\0';
+			h_tmp = (char *)realloc(h->req_buf, n + h->req_buflen + 1);
+			if (h_tmp == NULL)
+			{
+				syslog(LOG_WARNING, "Unable to allocate new memory for h->req_buf)");
+				h->state = EToDelete;
+			}
+			else
+			{
+				h->req_buf = h_tmp;
+				memcpy(h->req_buf + h->req_buflen, buf, n);
+				h->req_buflen += n;
+				h->req_buf[h->req_buflen] = '\0';
+			}
 			/* search for the string "\r\n\r\n" */
 			endheaders = findendheaders(h->req_buf, h->req_buflen);
 			if(endheaders)
 			{
+				/* at this point, the request buffer (h->req_buf)
+				 * is guaranteed to contain the \r\n\r\n character sequence */
 				h->req_contentoff = endheaders - h->req_buf + 4;
 				ProcessHttpQuery_upnphttp(h);
 			}
@@ -602,6 +755,10 @@ Process_upnphttp(struct upnphttp * h)
 			}
 		}
 		break;
+	case ESendingContinue:
+		if(SendResp_upnphttp(h))
+			h->state = EWaitingForHttpContent;
+		break;
 	case ESendingAndClosing:
 		SendRespAndClose_upnphttp(h);
 		break;
@@ -612,11 +769,11 @@ Process_upnphttp(struct upnphttp * h)
 
 static const char httpresphead[] =
 	"%s %d %s\r\n"
-	/*"Content-Type: text/xml; charset=\"utf-8\"\r\n"*/
 	"Content-Type: %s\r\n"
 	"Connection: close\r\n"
 	"Content-Length: %d\r\n"
 	"Server: " MINIUPNPD_SERVER_STRING "\r\n"
+	"Ext:\r\n"
 	;	/*"\r\n";*/
 /*
 		"<?xml version=\"1.0\"?>\n"
@@ -636,23 +793,42 @@ BuildHeader_upnphttp(struct upnphttp * h, int respcode,
                      int bodylen)
 {
 	int templen;
-	if(!h->res_buf)
-	{
-		templen = sizeof(httpresphead) + 128 + bodylen;
+	if(!h->res_buf ||
+	   h->res_buf_alloclen < ((int)sizeof(httpresphead) + 256 + bodylen)) {
+		if(h->res_buf)
+			free(h->res_buf);
+		templen = sizeof(httpresphead) + 256 + bodylen;
 		h->res_buf = (char *)malloc(templen);
-		if(!h->res_buf)
-		{
+		if(!h->res_buf) {
 			syslog(LOG_ERR, "malloc error in BuildHeader_upnphttp()");
 			return;
 		}
 		h->res_buf_alloclen = templen;
 	}
+	h->res_sent = 0;
 	h->res_buflen = snprintf(h->res_buf, h->res_buf_alloclen,
 	                         httpresphead, h->HttpVer,
 	                         respcode, respmsg,
-	                         (h->respflags&FLAG_HTML)?"text/html":"text/xml",
+	                         (h->respflags&FLAG_HTML)?"text/html":"text/xml; charset=\"utf-8\"",
 							 bodylen);
+	/* Content-Type MUST be 'text/xml; charset="utf-8"' according to UDA v1.1 */
+	/* Content-Type MUST be 'text/xml' according to UDA v1.0 */
 	/* Additional headers */
+#ifdef ENABLE_HTTP_DATE
+	{
+		char http_date[64];
+		time_t t;
+		struct tm tm;
+		time(&t);
+		gmtime_r(&t, &tm);
+		/* %a and %b depend on locale */
+		strftime(http_date, sizeof(http_date),
+		         "%a, %d %b %Y %H:%M:%S GMT", &tm);
+		h->res_buflen += snprintf(h->res_buf + h->res_buflen,
+		                          h->res_buf_alloclen - h->res_buflen,
+		                          "Date: %s\r\n", http_date);
+	}
+#endif
 #ifdef ENABLE_EVENTS
 	if(h->respflags & FLAG_TIMEOUT) {
 		h->res_buflen += snprintf(h->res_buf + h->res_buflen,
@@ -674,6 +850,22 @@ BuildHeader_upnphttp(struct upnphttp * h, int respcode,
 		                          "SID: %s\r\n", h->res_SID);
 	}
 #endif
+	if(h->respflags & FLAG_ALLOW_POST) {
+		h->res_buflen += snprintf(h->res_buf + h->res_buflen,
+		                          h->res_buf_alloclen - h->res_buflen,
+		                          "Allow: %s\r\n", "POST");
+	} else if(h->respflags & FLAG_ALLOW_SUB_UNSUB) {
+		h->res_buflen += snprintf(h->res_buf + h->res_buflen,
+		                          h->res_buf_alloclen - h->res_buflen,
+		                          "Allow: %s\r\n", "SUBSCRIBE, UNSUBSCRIBE");
+	}
+	if(h->accept_language[0] != '\0') {
+		/* defaulting to "en" */
+		h->res_buflen += snprintf(h->res_buf + h->res_buflen,
+		                          h->res_buf_alloclen - h->res_buflen,
+		                          "Content-Language: %s\r\n",
+		                          h->accept_language[0] == '*' ? "en" : h->accept_language);
+	}
 	h->res_buf[h->res_buflen++] = '\r';
 	h->res_buf[h->res_buflen++] = '\n';
 	if(h->res_buf_alloclen < (h->res_buflen + bodylen))
@@ -711,6 +903,41 @@ BuildResp_upnphttp(struct upnphttp * h,
 	BuildResp2_upnphttp(h, 200, "OK", body, bodylen);
 }
 
+int
+SendResp_upnphttp(struct upnphttp * h)
+{
+	ssize_t n;
+
+	while (h->res_sent < h->res_buflen)
+	{
+		n = send(h->socket, h->res_buf + h->res_sent,
+		         h->res_buflen - h->res_sent, 0);
+		if(n<0)
+		{
+			if(errno == EINTR)
+				continue;	/* try again immediatly */
+			if(errno == EAGAIN || errno == EWOULDBLOCK)
+			{
+				/* try again later */
+				return 0;
+			}
+			syslog(LOG_ERR, "send(res_buf): %m");
+			break; /* avoid infinite loop */
+		}
+		else if(n == 0)
+		{
+			syslog(LOG_ERR, "send(res_buf): %d bytes sent (out of %d)",
+							h->res_sent, h->res_buflen);
+			break;
+		}
+		else
+		{
+			h->res_sent += n;
+		}
+	}
+	return 1;	/* finished */
+}
+
 void
 SendRespAndClose_upnphttp(struct upnphttp * h)
 {
