@@ -1,12 +1,12 @@
-/* $Id: upnputils.h,v 1.2 2012/02/06 16:21:24 nanard Exp $ */
+/* $Id: upnputils.h,v 1.4 2013/02/06 10:50:04 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2011-2012 Thomas Bernard
+ * (c) 2011-2013 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
-#ifndef __UPNPUTILS_H__
-#define __UPNPUTILS_H__
+#ifndef UPNPUTILS_H_INCLUDED
+#define UPNPUTILS_H_INCLUDED
 
 /**
  * convert a struct sockaddr to a human readable string.
@@ -23,5 +23,11 @@ sockaddr_to_string(const struct sockaddr * addr, char * str, size_t size);
 int
 set_non_blocking(int fd);
 
+/**
+ * get the LAN which the peer belongs to
+ */
+struct lan_addr_s *
+get_lan_for_peer(const struct sockaddr * peer);
+
 #endif
 
